/**
 * Copyright 2012 Julien Eluard
 * This project includes software developed by Julien Eluard: https://github.com/jeluard/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.github.jeluard.guayaba.util.concurrent;

import com.google.common.base.Preconditions;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Helper methods for {@link ExecutorService}.
 */
public final class ExecutorServices {

  private ExecutorServices() {
  }

  /**
   * Force termination of specified {@link ExecutorService} in up to `timeout`.
   * <br />
   * First initiate shutdown. Give half `timeout` to finish. If not finished, force a {@link ExecutorService#shutdownNow()}.
   * Give yet another half `timeout` to finish
   *
   * @param executorService 
   * @param timeout 
   * @see ExecutorService#shutdown() 
   * @see ExecutorService#awaitTermination(long, java.util.concurrent.TimeUnit) 
   * @see ExecutorService#shutdownNow() 
   */
  public static void shutdownAndAwaitTermination(final ExecutorService executorService, final long timeout, final TimeUnit unit, final Logger logger) {
    Preconditions.checkNotNull(executorService, "null executorService");
    Preconditions.checkArgument(timeout > 0, "non positive timeout");
    Preconditions.checkNotNull(unit, "null unit");

    executorService.shutdown();
    final long halfTimeoutNanos = unit.toNanos(timeout) / 2;
    try {
      if (!executorService.awaitTermination(halfTimeoutNanos, TimeUnit.NANOSECONDS)) {
        ExecutorServices.shutdownNowAndAwaitTermination(executorService, halfTimeoutNanos, logger);
      }
    } catch (InterruptedException e) {
      //Force shutdownNow if interrupted before and exit quickly.
      executorService.shutdownNow();
      // Preserve interrupt status
      Thread.currentThread().interrupt();
    }
  }

  private static void shutdownNowAndAwaitTermination(final ExecutorService executorService, final long timeoutNanos, final Logger logger) throws InterruptedException {
    executorService.shutdownNow();
    if (!executorService.awaitTermination(timeoutNanos, TimeUnit.NANOSECONDS)) {
      if (logger.isLoggable(Level.WARNING)) {
        logger.log(Level.WARNING, "Failed to shutdown <{0}>, associated threads might not be freed.", executorService);
      }
    }
  }

}